#define IITUNES_NAME "IiTunes"
#define IITUNES_ID 5050

#define IITTRACK_NAME "IITTrack"
#define IITTRACK_ID 5051

#define IITTRACKCOLLECTION_NAME "IITTrackCollection"
#define IITTRACKCOLLECTION_ID 5052

#define IITPLAYLIST_NAME "IITPlaylist"
#define IITPLAYLIST_ID 5053

#define IITPLAYLISTCOLLECTION_NAME "IITPlaylistCollection"
#define IITPLAYLISTCOLLECTION_ID 5054

#define IITSOURCE_NAME "IITSource"
#define IITSOURCE_ID 5055

#define IITSOURCECOLLECTION_NAME "IITSourceCollection"
#define IITSOURCECOLLECTION_ID 5056

#define VALID_META (gLua->GetType(1) >= 5050) && (gLua->GetType(1) <= 5056)

#define gLua_PushBSTR(bstr) gLua->Push(_bstr_t(bstr))

#include <GMLuaModule.h>
#include <iTunesCOMInterface.h>

#include <atlbase.h>
#include <comdef.h>

GMOD_MODULE(Load, Unload);

ILuaInterface *gLua = NULL;

namespace IDispatchMethods
{
	LUA_FUNCTION(Release)
	{
		if (VALID_META)
		{
			IDispatch *iDispatch = (IDispatch*)gLua->GetUserData(1);
			
			iDispatch->Release();
		}

		return 0;
	}
}

namespace IITObjectMethods
{
	LUA_FUNCTION(GetIndex)
	{
		if (VALID_META)
		{
			IITObject *iObject = (IITObject*)gLua->GetUserData(1);

			long index;

			if (iObject->get_Index(&index) == S_OK)
				gLua->PushLong(index);
			else
				gLua->PushNil();
		}
		else
			gLua->PushNil();

		return 1;
	}

	LUA_FUNCTION(GetName)
	{
		if (VALID_META)
		{
			IITObject *iObject = (IITObject*)gLua->GetUserData(1);

			CComBSTR name;

			if (iObject->get_Name(&name) == S_OK)
				gLua_PushBSTR(name);
			else
				gLua->PushNil();
		}
		else
			gLua->PushNil();

		return 1;	
	}

	LUA_FUNCTION(GetPlaylistID)
	{
		if (VALID_META)
		{
			IITObject *iObject = (IITObject*)gLua->GetUserData(1);

			long playlistID;

			if (iObject->get_PlaylistID(&playlistID) == S_OK)
				gLua->PushLong(playlistID);
			else
				gLua->PushNil();
		}
		else
			gLua->PushNil();

		return 1;		
	}

	LUA_FUNCTION(GetSourceID)
	{
		if (VALID_META)
		{
			IITObject *iObject = (IITObject*)gLua->GetUserData(1);

			long sourceID;

			if (iObject->get_SourceID(&sourceID) == S_OK)
				gLua->PushLong(sourceID);
			else
				gLua->PushNil();
		}
		else
			gLua->PushNil();

		return 1;	
	}

	LUA_FUNCTION(GetTrackDatabaseID)
	{
		if (VALID_META)
		{
			IITObject *iObject = (IITObject*)gLua->GetUserData(1);

			long trackDatabaseID;

			if (iObject->get_TrackDatabaseID(&trackDatabaseID) == S_OK)
				gLua->PushLong(trackDatabaseID);
			else
				gLua->PushNil();
		}
		else
			gLua->PushNil();

		return 1;	
	}

	LUA_FUNCTION(GetTrackID)
	{
		if (VALID_META)
		{
			IITObject *iObject = (IITObject*)gLua->GetUserData(1);

			long trackID;

			if (iObject->get_TrackID(&trackID) == S_OK)
				gLua->PushLong(trackID);
			else
				gLua->PushNil();
		}
		else
			gLua->PushNil();

		return 0;
	}

	LUA_FUNCTION(SetName)
	{
		if (VALID_META)
		{
			gLua->CheckType(2, GLua::TYPE_STRING);

			IITObject *iObject = (IITObject*)gLua->GetUserData(1);

			CComBSTR name(gLua->GetString(2));

			if (iObject->put_Name(name) == S_OK)
				gLua_PushBSTR(name);
			else
				gLua->Push(false);
		}
		else
			gLua->PushNil();

		return 1;
	}
}

namespace IiTunesMethods
{
	LUA_FUNCTION(BackTrack)
	{
		gLua->CheckType(1, IITUNES_ID);

		IiTunes *iITunes = (IiTunes*)gLua->GetUserData(1);

		if (iITunes->BackTrack() == S_OK)
			gLua->Push(true);
		else
			gLua->Push(false);

		return 1;
	}

	LUA_FUNCTION(FastForward)
	{
		gLua->CheckType(1, IITUNES_ID);

		IiTunes *iITunes = (IiTunes*)gLua->GetUserData(1);
		
		if (iITunes->FastForward() == S_OK)
			gLua->Push(true);
		else
			gLua->Push(false);

		return 1;
	}

	LUA_FUNCTION(GetCurrentPlaylist)
	{
		gLua->CheckType(1, IITUNES_ID);
		
		IiTunes *iITunes = (IiTunes*)gLua->GetUserData(1);

		IITPlaylist *iPlaylist = NULL;

		if (iITunes->get_CurrentPlaylist(&iPlaylist) == S_OK)
			gLua->PushUserData(gLua->GetMetaTable(IITPLAYLIST_NAME, IITPLAYLIST_ID), iPlaylist);
		else
			gLua->PushNil();

		return 1;
	}

	LUA_FUNCTION(GetCurrentStreamTitle)
	{
		gLua->CheckType(1, IITUNES_ID);
		
		IiTunes *iITunes = (IiTunes*)gLua->GetUserData(1);

		CComBSTR streamTitle;

		if (iITunes->get_CurrentStreamTitle(&streamTitle) == S_OK)
			gLua_PushBSTR(streamTitle);
		else
			gLua->PushNil();

		return 1;
	}

	LUA_FUNCTION(GetCurrentStreamURL)
	{
		gLua->CheckType(1, IITUNES_ID);
		
		IiTunes *iITunes = (IiTunes*)gLua->GetUserData(1);

		CComBSTR streamURL;

		if (iITunes->get_CurrentStreamURL(&streamURL) == S_OK)
			gLua_PushBSTR(streamURL);
		else
			gLua->PushNil();

		return 1;
	}

	LUA_FUNCTION(GetCurrentTrack)
	{
		gLua->CheckType(1, IITUNES_ID);

		IiTunes *iITunes = (IiTunes*)gLua->GetUserData(1);

		IITTrack *iTrack = NULL;

		if (iITunes->get_CurrentTrack(&iTrack) == S_OK)
			gLua->PushUserData(gLua->GetMetaTable(IITTRACK_NAME, IITTRACK_ID), iTrack);
		else
			gLua->PushNil();

		return 1;
	}

	LUA_FUNCTION(GetLibraryPlaylist)
	{
		gLua->CheckType(1, IITUNES_ID);

		IiTunes *iITunes = (IiTunes*)gLua->GetUserData(1);

		IITLibraryPlaylist *iLibraryPlaylist = NULL;
		
		if (iITunes->get_LibraryPlaylist(&iLibraryPlaylist) == S_OK)
			gLua->PushUserData(gLua->GetMetaTable(IITPLAYLIST_NAME, IITPLAYLIST_ID), (IITPlaylist*)iLibraryPlaylist);
		else
			gLua->PushNil();

		return 1;
	}

	LUA_FUNCTION(GetLibrarySource)
	{
		gLua->CheckType(1, IITUNES_ID);

		IiTunes *iITunes = (IiTunes*)gLua->GetUserData(1);

		IITSource *iLibrarySource = NULL;

		if (iITunes->get_LibrarySource(&iLibrarySource) == S_OK)
			gLua->PushUserData(gLua->GetMetaTable(IITSOURCE_NAME, IITSOURCE_ID), iLibrarySource);
		else
			gLua->PushNil();

		return 1;
	}

	LUA_FUNCTION(GetMute)
	{
		gLua->CheckType(1, IITUNES_ID);

		IiTunes *iITunes = (IiTunes*)gLua->GetUserData(1);

		short isMuted;

		if (iITunes->get_Mute(&isMuted) == S_OK)
			gLua->Push((isMuted == VARIANT_TRUE));
		else
			gLua->PushNil();

		return 1;
	}
	
	LUA_FUNCTION(GetPlayerPosition)
	{
		gLua->CheckType(1, IITUNES_ID);

		IiTunes *iITunes = (IiTunes*)gLua->GetUserData(1);

		long playerPos;

		if (iITunes->get_PlayerPosition(&playerPos) == S_OK)
			gLua->PushLong(playerPos);
		else
			gLua->PushNil();

		return 1;
	}

	LUA_FUNCTION(GetPlayerState)
	{
		gLua->CheckType(1, IITUNES_ID);

		IiTunes *iITunes = (IiTunes*)gLua->GetUserData(1);

		ITPlayerState playerState;

		if (iITunes->get_PlayerState(&playerState) == S_OK)
			gLua->PushDouble(playerState);
		else
			gLua->PushNil();

		return 1;
	}
	
	LUA_FUNCTION(GetSoundVolume)
	{
		gLua->CheckType(1, IITUNES_ID);

		IiTunes *iITunes = (IiTunes*)gLua->GetUserData(1);

		long volume;

		if (iITunes->get_SoundVolume(&volume) == S_OK)
			gLua->PushLong(volume);
		else
			gLua->PushNil();

		return 1;
	}
	
	LUA_FUNCTION(GetSources)
	{
		gLua->CheckType(1, IITUNES_ID);
		
		IiTunes *iITunes = (IiTunes*)gLua->GetUserData(1);

		IITSourceCollection *iSourceCollection = NULL;

		if (iITunes->get_Sources(&iSourceCollection) == S_OK)
			gLua->PushUserData(gLua->GetMetaTable(IITSOURCECOLLECTION_NAME, IITSOURCECOLLECTION_ID), iSourceCollection);
		else
			gLua->PushNil();

		return 1;
	}

	LUA_FUNCTION(GetVersion)
	{
		gLua->CheckType(1, IITUNES_ID);

		IiTunes *iITunes = (IiTunes*)gLua->GetUserData(1);

		CComBSTR version;

		if (iITunes->get_Version(&version) == S_OK)
			gLua_PushBSTR(version);
		else
			gLua->PushNil();

		return 1;
	}
	
	LUA_FUNCTION(NextTrack)
	{
		gLua->CheckType(1, IITUNES_ID);

		IiTunes *iITunes = (IiTunes*)gLua->GetUserData(1);
		
		if (iITunes->NextTrack() == S_OK)
			gLua->Push(true);
		else
			gLua->Push(false);

		return 1;
	}
	
	LUA_FUNCTION(Pause)
	{
		gLua->CheckType(1, IITUNES_ID);

		IiTunes *iITunes = (IiTunes*)gLua->GetUserData(1);
		
		if (iITunes->Pause() == S_OK)
			gLua->Push(true);
		else
			gLua->Push(false);

		return 1;
	}
	
	LUA_FUNCTION(Play)
	{
		gLua->CheckType(1, IITUNES_ID);

		IiTunes *iITunes = (IiTunes*)gLua->GetUserData(1);
		
		if (iITunes->Play() == S_OK)
			gLua->Push(true);
		else
			gLua->Push(false);

		return 1;
	}

	LUA_FUNCTION(PlayFile)
	{
		gLua->CheckType(1, IITUNES_ID);
		gLua->CheckType(2, GLua::TYPE_STRING);

		IiTunes *iITunes = (IiTunes*)gLua->GetUserData(1);
		
		CComBSTR filePath(gLua->GetString(2));

		if (iITunes->PlayFile(filePath) == S_OK)
			gLua->Push(true);
		else
			gLua->Push(false);

		return 1;
	}

	LUA_FUNCTION(PlayPause)
	{
		gLua->CheckType(1, IITUNES_ID);

		IiTunes *iITunes = (IiTunes*)gLua->GetUserData(1);
		
		if (iITunes->PlayPause() == S_OK)
			gLua->Push(true);
		else
			gLua->Push(false);

		return 1;
	}

	LUA_FUNCTION(PreviousTrack)
	{
		gLua->CheckType(1, IITUNES_ID);

		IiTunes *iITunes = (IiTunes*)gLua->GetUserData(1);
		
		if (iITunes->PreviousTrack() == S_OK)
			gLua->Push(true);
		else
			gLua->Push(false);

		return 1;
	}
	
	LUA_FUNCTION(Quit)
	{
		gLua->CheckType(1, IITUNES_ID);

		IiTunes *iITunes = (IiTunes*)gLua->GetUserData(1);
		
		if (iITunes->Quit() == S_OK)
			gLua->Push(true);
		else
			gLua->Push(false);

		return 1;
	}
	
	LUA_FUNCTION(Resume)
	{
		gLua->CheckType(1, IITUNES_ID);

		IiTunes *iITunes = (IiTunes*)gLua->GetUserData(1);
		
		if (iITunes->Resume() == S_OK)
			gLua->Push(true);
		else
			gLua->Push(false);

		return 1;
	}

	LUA_FUNCTION(Rewind)
	{
		gLua->CheckType(1, IITUNES_ID);

		IiTunes *iITunes = (IiTunes*)gLua->GetUserData(1);
		
		if (iITunes->Rewind() == S_OK)
			gLua->Push(true);
		else
			gLua->Push(false);

		return 1;
	}

	LUA_FUNCTION(SetMute)
	{
		gLua->CheckType(1, IITUNES_ID);
		gLua->CheckType(2, GLua::TYPE_BOOL);

		IiTunes *iITunes = (IiTunes*)gLua->GetUserData(1);

		short shouldMute = VARIANT_FALSE;

		if (gLua->GetBool(2))
			shouldMute = VARIANT_TRUE;

		if (iITunes->put_Mute(shouldMute) == S_OK)
			gLua->Push(true);
		else
			gLua->Push(false);

		return 1;
	}

	LUA_FUNCTION(SetPlayerPosition)
	{
		gLua->CheckType(1, IITUNES_ID);

		IiTunes *iITunes = (IiTunes*)gLua->GetUserData(1);

		if (iITunes->put_PlayerPosition(gLua->GetInteger(2)) == S_OK)
			gLua->Push(true);
		else
			gLua->Push(false);

		return 1;
	}
	
	LUA_FUNCTION(SetSoundVolume)
	{
		gLua->CheckType(1, IITUNES_ID);
		gLua->CheckType(2, GLua::TYPE_NUMBER);

		IiTunes *iITunes = (IiTunes*)gLua->GetUserData(1);

		if (iITunes->put_SoundVolume(gLua->GetInteger(2)) == S_OK)
			gLua->Push(true);
		else
			gLua->Push(false);

		return 1;
	}
	
	LUA_FUNCTION(Stop)
	{
		gLua->CheckType(1, IITUNES_ID);

		IiTunes *iITunes = (IiTunes*)gLua->GetUserData(1);

		if (iITunes->Stop() == S_OK)
			gLua->Push(true);
		else
			gLua->Push(false);

		return 1;
	}

	LUA_FUNCTION(SubscribeToPodcast)
	{
		gLua->CheckType(1, IITUNES_ID);
		gLua->CheckType(2, GLua::TYPE_STRING);

		IiTunes *iITunes = (IiTunes*)gLua->GetUserData(1);

		CComBSTR url(gLua->GetString(2));

		if (iITunes->SubscribeToPodcast(url) == S_OK)
			gLua->Push(true);
		else
			gLua->Push(false);

		return 1;
	}

	LUA_FUNCTION(UpdateIPod)
	{
		gLua->CheckType(1, IITUNES_ID);

		IiTunes *iITunes = (IiTunes*)gLua->GetUserData(1);

		if (iITunes->UpdateIPod() == S_OK)
			gLua->Push(true);
		else
			gLua->Push(false);

		return 1;
	}

	LUA_FUNCTION(UpdatePodcastFeeds)
	{
		gLua->CheckType(1, IITUNES_ID);

		IiTunes *iITunes = (IiTunes*)gLua->GetUserData(1);

		if (iITunes->UpdatePodcastFeeds() == S_OK)
			gLua->Push(true);
		else
			gLua->Push(false);

		return 1;
	}
}

namespace IITTrackMethods
{
	LUA_FUNCTION(Delete)
	{
		gLua->CheckType(1, IITTRACK_ID);

		IITTrack *iTrack = (IITTrack*)gLua->GetUserData(1);
		
		if (iTrack->Delete() == S_OK)
			gLua->Push(true);
		else
			gLua->Push(false);

		return 1;
	}

	LUA_FUNCTION(GetAlbum)
	{
		gLua->CheckType(1, IITTRACK_ID);

		IITTrack *iTrack = (IITTrack*)gLua->GetUserData(1);
	
		CComBSTR album;

		if (iTrack->get_Album(&album) == S_OK)
			gLua_PushBSTR(album);
		else
			gLua->PushNil();

		return 1;
	}

	LUA_FUNCTION(GetArtist)
	{
		gLua->CheckType(1, IITTRACK_ID);

		IITTrack *iTrack = (IITTrack*)gLua->GetUserData(1);
	
		CComBSTR artist;

		if (iTrack->get_Artist(&artist) == S_OK)
			gLua_PushBSTR(artist);
		else
			gLua->PushNil();
		
		return 1;
	}

	LUA_FUNCTION(GetBitRate)
	{
		gLua->CheckType(1, IITTRACK_ID);

		IITTrack *iTrack = (IITTrack*)gLua->GetUserData(1);
		
		long bitrate;

		if (iTrack->get_BitRate(&bitrate) == S_OK)
			gLua->PushLong(bitrate);
		else
			gLua->PushNil();

		return 1;
	}

	LUA_FUNCTION(GetBPM)
	{
		gLua->CheckType(1, IITTRACK_ID);

		IITTrack *iTrack = (IITTrack*)gLua->GetUserData(1);
		
		long beatsPerMinute;

		if (iTrack->get_BPM(&beatsPerMinute) == S_OK)
			gLua->PushLong(beatsPerMinute);
		else
			gLua->PushNil();
		
		return 1;
	}

	LUA_FUNCTION(GetComment)
	{
		gLua->CheckType(1, IITTRACK_ID);

		IITTrack *iTrack = (IITTrack*)gLua->GetUserData(1);
		
		CComBSTR comment;

		if (iTrack->get_Comment(&comment) == S_OK)
			gLua_PushBSTR(comment);
		else
			gLua->PushNil();

		return 1;
	}

	LUA_FUNCTION(GetCompilation)
	{
		gLua->CheckType(1, IITTRACK_ID);

		IITTrack *iTrack = (IITTrack*)gLua->GetUserData(1);
		
		short isCompilation;
		
		if (iTrack->get_Compilation(&isCompilation) == S_OK)
			gLua->Push((isCompilation == VARIANT_TRUE));
		else
			gLua->PushNil();

		return 1;
	}

	LUA_FUNCTION(GetComposer)
	{
		gLua->CheckType(1, IITTRACK_ID);

		IITTrack *iTrack = (IITTrack*)gLua->GetUserData(1);
		
		CComBSTR composer;

		if (iTrack->get_Comment(&composer) == S_OK)
			gLua_PushBSTR(composer);
		else
			gLua->PushNil();

		return 1;
	}

	LUA_FUNCTION(GetDateAdded)
	{
		gLua->CheckType(1, IITTRACK_ID);

		IITTrack *iTrack = (IITTrack*)gLua->GetUserData(1);
		
		double dateAdded;

		if (iTrack->get_DateAdded(&dateAdded) == S_OK)
			gLua->PushDouble(dateAdded);
		else
			gLua->PushNil();

		return 1;
	}

	LUA_FUNCTION(GetDiscCount)
	{
		gLua->CheckType(1, IITTRACK_ID);

		IITTrack *iTrack = (IITTrack*)gLua->GetUserData(1);
		
		long discCount;

		if (iTrack->get_DiscCount(&discCount) == S_OK)
			gLua->PushLong(discCount);
		else
			gLua->PushNil();

		return 1;
	}

	LUA_FUNCTION(GetDiscNumber)
	{
		gLua->CheckType(1, IITTRACK_ID);

		IITTrack *iTrack = (IITTrack*)gLua->GetUserData(1);
		
		long discNumber;

		if (iTrack->get_DiscNumber(&discNumber) == S_OK)
			gLua->PushLong(discNumber);
		else
			gLua->PushNil();

		return 1;
	}
	
	LUA_FUNCTION(GetDuration)
	{
		gLua->CheckType(1, IITTRACK_ID);

		IITTrack *iTrack = (IITTrack*)gLua->GetUserData(1);
		
		long duration;

		if (iTrack->get_Duration(&duration) == S_OK)
			gLua->PushLong(duration);
		else
			gLua->PushNil();

		return 1;
	}

	LUA_FUNCTION(GetEnabled)
	{
		gLua->CheckType(1, IITTRACK_ID);

		IITTrack *iTrack = (IITTrack*)gLua->GetUserData(1);
		
		short isEnabled;

		if (iTrack->get_Enabled(&isEnabled) == S_OK)
			gLua->Push((isEnabled == VARIANT_TRUE));
		else
			gLua->PushNil();

		return 1;
	}

	LUA_FUNCTION(GetEQ)
	{
		gLua->CheckType(1, IITTRACK_ID);

		IITTrack *iTrack = (IITTrack*)gLua->GetUserData(1);
		
		CComBSTR eq;

		if (iTrack->get_EQ(&eq) == S_OK)
			gLua_PushBSTR(eq);
		else
			gLua->PushNil();

		return 1;
	}

	LUA_FUNCTION(GetFinish)
	{
		gLua->CheckType(1, IITTRACK_ID);

		IITTrack *iTrack = (IITTrack*)gLua->GetUserData(1);
		
		long finish;

		if (iTrack->get_Finish(&finish) == S_OK)
			gLua->PushLong(finish);
		else
			gLua->PushNil();

		return 1;
	}

	LUA_FUNCTION(GetGenre)
	{
		gLua->CheckType(1, IITTRACK_ID);

		IITTrack *iTrack = (IITTrack*)gLua->GetUserData(1);
	
		CComBSTR genre;

		if (iTrack->get_Genre(&genre) == S_OK)
			gLua_PushBSTR(genre);
		else
			gLua->PushNil();
		
		return 1;
	}

	LUA_FUNCTION(GetGrouping)
	{
		gLua->CheckType(1, IITTRACK_ID);

		IITTrack *iTrack = (IITTrack*)gLua->GetUserData(1);
		
		CComBSTR grouping;

		if (iTrack->get_Grouping(&grouping) == S_OK)
			gLua_PushBSTR(grouping);
		else
			gLua->PushNil();

		return 1;
	}

	LUA_FUNCTION(GetKind)
	{
		gLua->CheckType(1, IITTRACK_ID);

		IITTrack *iTrack = (IITTrack*)gLua->GetUserData(1);
		
		ITTrackKind kind;

		if (iTrack->get_Kind(&kind) == S_OK)
			gLua->PushDouble(kind);
		else
			gLua->PushNil();

		return 1;
	}

	LUA_FUNCTION(GetModificationDate)
	{
		gLua->CheckType(1, IITTRACK_ID);

		IITTrack *iTrack = (IITTrack*)gLua->GetUserData(1);
		
		double dateModified;

		if (iTrack->get_ModificationDate(&dateModified) == S_OK)
			gLua->PushDouble(dateModified);
		else
			gLua->PushNil();

		return 1;
	}

	LUA_FUNCTION(GetPlayedCount)
	{
		gLua->CheckType(1, IITTRACK_ID);

		IITTrack *iTrack = (IITTrack*)gLua->GetUserData(1);

		long playedCount;

		if (iTrack->get_PlayedCount(&playedCount) == S_OK)
			gLua->PushLong(playedCount);
		else
			gLua->PushNil();

		return 1;
	}

	LUA_FUNCTION(GetPlayedDate)
	{
		gLua->CheckType(1, IITTRACK_ID);

		IITTrack *iTrack = (IITTrack*)gLua->GetUserData(1);
		
		double playedDate;

		if (iTrack->get_PlayedDate(&playedDate) == S_OK)
			gLua->PushDouble(playedDate);
		else
			gLua->PushNil();

		return 1;
	}

	LUA_FUNCTION(GetPlaylist)
	{
		gLua->CheckType(1, IITTRACK_ID);

		IITTrack *iTrack = (IITTrack*)gLua->GetUserData(1);
		
		IITPlaylist *iPlaylist = NULL;

		if (iTrack->get_Playlist(&iPlaylist) == S_OK)
			gLua->PushUserData(gLua->GetMetaTable(IITPLAYLIST_NAME, IITPLAYLIST_ID), iPlaylist);
		else
			gLua->PushNil();

		return 1;
	}
	
	LUA_FUNCTION(GetPlayOrderIndex)
	{
		gLua->CheckType(1, IITTRACK_ID);

		IITTrack *iTrack = (IITTrack*)gLua->GetUserData(1);
		
		long index;

		if (iTrack->get_PlayOrderIndex(&index) == S_OK)
			gLua->PushLong(index);
		else
			gLua->PushNil();

		return 1;
	}

	LUA_FUNCTION(GetRating)
	{
		gLua->CheckType(1, IITTRACK_ID);

		IITTrack *iTrack = (IITTrack*)gLua->GetUserData(1);

		long rating;

		if (iTrack->get_Rating(&rating) == S_OK)
			gLua->PushLong(rating);
		else
			gLua->PushNil();

		return 1;
	}

	LUA_FUNCTION(GetSampleRate)
	{
		gLua->CheckType(1, IITTRACK_ID);

		IITTrack *iTrack = (IITTrack*)gLua->GetUserData(1);
		
		long sampleRate;

		if (iTrack->get_SampleRate(&sampleRate) == S_OK)
			gLua->PushLong(sampleRate);
		else
			gLua->PushNil();

		return 1;
	}

	LUA_FUNCTION(GetSize)
	{
		gLua->CheckType(1, IITTRACK_ID);

		IITTrack *iTrack = (IITTrack*)gLua->GetUserData(1);
		
		long size;

		if (iTrack->get_Size(&size) == S_OK)
			gLua->PushLong(size);
		else
			gLua->PushNil();

		return 1;
	}

	LUA_FUNCTION(GetStart)
	{
		gLua->CheckType(1, IITTRACK_ID);

		IITTrack *iTrack = (IITTrack*)gLua->GetUserData(1);
		
		long start;

		if (iTrack->get_Start(&start) == S_OK)
			gLua->PushLong(start);
		else
			gLua->PushNil();

		return 1;
	}

	LUA_FUNCTION(GetTime)
	{
		gLua->CheckType(1, IITTRACK_ID);

		IITTrack *iTrack = (IITTrack*)gLua->GetUserData(1);

		CComBSTR time;

		if (iTrack->get_Time(&time) == S_OK)
			gLua_PushBSTR(time);
		else
			gLua->PushNil();

		return 1;
	}

	LUA_FUNCTION(GetTrackCount)
	{
		gLua->CheckType(1, IITTRACK_ID);

		IITTrack *iTrack = (IITTrack*)gLua->GetUserData(1);
	
		long trackCount;

		if (iTrack->get_TrackCount(&trackCount) == S_OK)
			gLua->PushLong(trackCount);
		else
			gLua->PushNil();

		return 1;
	}

	LUA_FUNCTION(GetTrackNumber)
	{
		gLua->CheckType(1, IITTRACK_ID);

		IITTrack *iTrack = (IITTrack*)gLua->GetUserData(1);
		
		long trackNumber;

		if (iTrack->get_TrackNumber(&trackNumber) == S_OK)
			gLua->PushLong(trackNumber);
		else
			gLua->PushNil();

		return 1;
	}

	LUA_FUNCTION(GetVolumeAdjustment)
	{
		gLua->CheckType(1, IITTRACK_ID);

		IITTrack *iTrack = (IITTrack*)gLua->GetUserData(1);
		
		long volumeAdjustment;

		if (iTrack->get_VolumeAdjustment(&volumeAdjustment) == S_OK)
			gLua->PushLong(volumeAdjustment);
		else
			gLua->PushNil();

		return 1;
	}

	LUA_FUNCTION(GetYear)
	{
		gLua->CheckType(1, IITTRACK_ID);

		IITTrack *iTrack = (IITTrack*)gLua->GetUserData(1);
		
		long year;

		if (iTrack->get_Year(&year) == S_OK)
			gLua->PushLong(year);
		else
			gLua->PushNil();

		return 1;
	}

	LUA_FUNCTION(Play)
	{
		gLua->CheckType(1, IITTRACK_ID);

		IITTrack *iTrack = (IITTrack*)gLua->GetUserData(1);

		if (iTrack->Play() == S_OK)
			gLua->Push(true);
		else
			gLua->Push(false);

		return 1;
	}

	LUA_FUNCTION(SetAlbum)
	{
		gLua->CheckType(1, IITTRACK_ID);
		gLua->CheckType(2, GLua::TYPE_STRING);

		IITTrack *iTrack = (IITTrack*)gLua->GetUserData(1);
		
		CComBSTR album(gLua->GetString(2));

		if (iTrack->put_Album(album) == S_OK)
			gLua->Push(true);
		else
			gLua->Push(false);

		return 1;
	}

	LUA_FUNCTION(SetArtist)
	{
		gLua->CheckType(1, IITTRACK_ID);
		gLua->CheckType(2, GLua::TYPE_STRING);

		IITTrack *iTrack = (IITTrack*)gLua->GetUserData(1);
		
		CComBSTR artist(gLua->GetString(2));

		if (iTrack->put_Artist(artist) == S_OK)
			gLua->Push(true);
		else
			gLua->Push(false);

		return 1;
	}

	LUA_FUNCTION(SetBPM)
	{
		gLua->CheckType(1, IITTRACK_ID);
		gLua->CheckType(2, GLua::TYPE_NUMBER);

		IITTrack *iTrack = (IITTrack*)gLua->GetUserData(1);
		
		if (iTrack->put_BPM(gLua->GetInteger(2)) == S_OK)
			gLua->Push(true);
		else
			gLua->Push(false);

		return 1;
	}

	LUA_FUNCTION(SetComment)
	{
		gLua->CheckType(1, IITTRACK_ID);
		gLua->CheckType(2, GLua::TYPE_STRING);

		IITTrack *iTrack = (IITTrack*)gLua->GetUserData(1);
		
		CComBSTR comment(gLua->GetString(2));

		if (iTrack->put_Comment(comment) == S_OK)
			gLua->Push(true);
		else
			gLua->Push(false);

		return 1;
	}

	LUA_FUNCTION(SetCompilation)
	{
		gLua->CheckType(1, IITTRACK_ID);
		gLua->CheckType(2, GLua::TYPE_BOOL);

		IITTrack *iTrack = (IITTrack*)gLua->GetUserData(1);
		
		short shouldBeCompilation = VARIANT_FALSE;

		if (gLua->GetBool(2))
			shouldBeCompilation = VARIANT_TRUE;

		if (iTrack->put_Compilation(shouldBeCompilation) == S_OK)
			gLua->Push(true);
		else
			gLua->Push(false);

		return 1;
	}

	LUA_FUNCTION(SetComposer)
	{
		gLua->CheckType(1, IITTRACK_ID);
		gLua->CheckType(2, GLua::TYPE_STRING);

		IITTrack *iTrack = (IITTrack*)gLua->GetUserData(1);
		
		CComBSTR composer(gLua->GetString(2));

		if (iTrack->put_Composer(composer) == S_OK)
			gLua->Push(true);
		else
			gLua->Push(false);

		return 1;
	}

	LUA_FUNCTION(SetDiscCount)
	{
		gLua->CheckType(1, IITTRACK_ID);
		gLua->CheckType(2, GLua::TYPE_NUMBER);

		IITTrack *iTrack = (IITTrack*)gLua->GetUserData(1);
		
		if (iTrack->put_DiscCount(gLua->GetInteger(2)) == S_OK)
			gLua->Push(true);
		else
			gLua->Push(false);

		return 1;
	}

	LUA_FUNCTION(SetDiscNumber)
	{
		gLua->CheckType(1, IITTRACK_ID);
		gLua->CheckType(2, GLua::TYPE_NUMBER);

		IITTrack *iTrack = (IITTrack*)gLua->GetUserData(1);
		
		if (iTrack->put_DiscNumber(gLua->GetInteger(2)) == S_OK)
			gLua->Push(true);
		else
			gLua->Push(false);

		return 1;
	}

	LUA_FUNCTION(SetEnabled)
	{
		gLua->CheckType(1, IITTRACK_ID);
		gLua->CheckType(2, GLua::TYPE_BOOL);

		IITTrack *iTrack = (IITTrack*)gLua->GetUserData(1);
		
		short shouldBeEnabled = VARIANT_FALSE;

		if (gLua->GetBool(2))
			shouldBeEnabled = VARIANT_TRUE;

		if (iTrack->put_Enabled(shouldBeEnabled) == S_OK)
			gLua->Push(true);
		else
			gLua->Push(false);

		return 1;
	}

	LUA_FUNCTION(SetEQ)
	{
		gLua->CheckType(1, IITTRACK_ID);
		gLua->CheckType(2, GLua::TYPE_STRING);

		IITTrack *iTrack = (IITTrack*)gLua->GetUserData(1);
		
		CComBSTR eq(gLua->GetString(2));

		if (iTrack->put_EQ(eq) == S_OK)
			gLua->Push(true);
		else
			gLua->Push(false);

		return 1;
	}

	LUA_FUNCTION(SetFinish)
	{
		gLua->CheckType(1, IITTRACK_ID);
		gLua->CheckType(2, GLua::TYPE_NUMBER);

		IITTrack *iTrack = (IITTrack*)gLua->GetUserData(1);
		
		if (iTrack->put_Finish(gLua->GetInteger(2)) == S_OK)
			gLua->Push(true);
		else
			gLua->Push(false);

		return 1;
	}

	LUA_FUNCTION(SetGenre)
	{
		gLua->CheckType(1, IITTRACK_ID);
		gLua->CheckType(2, GLua::TYPE_STRING);

		IITTrack *iTrack = (IITTrack*)gLua->GetUserData(1);
		
		CComBSTR genre(gLua->GetString(2));

		if (iTrack->put_Genre(genre) == S_OK)
			gLua->Push(true);
		else
			gLua->Push(false);

		return 1;
	}

	LUA_FUNCTION(SetGrouping)
	{
		gLua->CheckType(1, IITTRACK_ID);
		gLua->CheckType(2, GLua::TYPE_STRING);

		IITTrack *iTrack = (IITTrack*)gLua->GetUserData(1);
		
		CComBSTR grouping(gLua->GetString(2));

		if (iTrack->put_Grouping(grouping) == S_OK)
			gLua->Push(true);
		else
			gLua->Push(false);

		return 1;
	}

	LUA_FUNCTION(SetPlayedCount)
	{
		gLua->CheckType(1, IITTRACK_ID);
		gLua->CheckType(2, GLua::TYPE_NUMBER);

		IITTrack *iTrack = (IITTrack*)gLua->GetUserData(1);
		
		if (iTrack->put_PlayedCount(gLua->GetInteger(2)) == S_OK)
			gLua->Push(true);
		else
			gLua->Push(false);

		return 1;
	}

	LUA_FUNCTION(SetPlayedDate)
	{
		gLua->CheckType(1, IITTRACK_ID);
		gLua->CheckType(2, GLua::TYPE_NUMBER);

		IITTrack *iTrack = (IITTrack*)gLua->GetUserData(1);
		
		if (iTrack->put_PlayedDate(gLua->GetInteger(2)) == S_OK)
			gLua->Push(true);
		else
			gLua->Push(false);

		return 1;
	}

	LUA_FUNCTION(SetRating)
	{
		gLua->CheckType(1, IITTRACK_ID);
		gLua->CheckType(2, GLua::TYPE_NUMBER);

		IITTrack *iTrack = (IITTrack*)gLua->GetUserData(1);
		
		if (iTrack->put_Rating(gLua->GetInteger(2)) == S_OK)
			gLua->Push(true);
		else
			gLua->Push(false);

		return 1;
	}

	LUA_FUNCTION(SetStart)
	{
		gLua->CheckType(1, IITTRACK_ID);
		gLua->CheckType(2, GLua::TYPE_NUMBER);

		IITTrack *iTrack = (IITTrack*)gLua->GetUserData(1);
		
		if (iTrack->put_Start(gLua->GetInteger(2)) == S_OK)
			gLua->Push(true);
		else
			gLua->Push(false);

		return 1;
	}

	LUA_FUNCTION(SetTrackCount)
	{
		gLua->CheckType(1, IITTRACK_ID);
		gLua->CheckType(2, GLua::TYPE_NUMBER);

		IITTrack *iTrack = (IITTrack*)gLua->GetUserData(1);
		
		if (iTrack->put_TrackCount(gLua->GetInteger(2)) == S_OK)
			gLua->Push(true);
		else
			gLua->Push(false);

		return 1;
	}

	LUA_FUNCTION(SetTrackNumber)
	{
		gLua->CheckType(1, IITTRACK_ID);
		gLua->CheckType(2, GLua::TYPE_NUMBER);

		IITTrack *iTrack = (IITTrack*)gLua->GetUserData(1);
		
		if (iTrack->put_TrackNumber(gLua->GetInteger(2)) == S_OK)
			gLua->Push(true);
		else
			gLua->Push(false);

		return 1;
	}

	LUA_FUNCTION(SetVolumeAdjustment)
	{
		gLua->CheckType(1, IITTRACK_ID);
		gLua->CheckType(2, GLua::TYPE_NUMBER);

		IITTrack *iTrack = (IITTrack*)gLua->GetUserData(1);
		
		if (iTrack->put_VolumeAdjustment(gLua->GetInteger(2)) == S_OK)
			gLua->Push(true);
		else
			gLua->Push(false);

		return 1;
	}

	LUA_FUNCTION(SetYear)
	{
		gLua->CheckType(1, IITTRACK_ID);
		gLua->CheckType(2, GLua::TYPE_NUMBER);

		IITTrack *iTrack = (IITTrack*)gLua->GetUserData(1);
		
		if (iTrack->put_Year(gLua->GetInteger(2)) == S_OK)
			gLua->Push(true);
		else
			gLua->Push(false);

		return 1;
	}
}

namespace IITTrackCollectionMethods
{
	LUA_FUNCTION(GetCount)
	{
		gLua->CheckType(1, IITTRACKCOLLECTION_ID);

		IITTrackCollection *iTrackCollection = (IITTrackCollection*)gLua->GetUserData(1);

		long count;

		if (iTrackCollection->get_Count(&count) == S_OK)
			gLua->PushLong(count);
		else
			gLua->PushNil();

		return 1;
	}

	LUA_FUNCTION(GetItem)
	{
		gLua->CheckType(1, IITTRACKCOLLECTION_ID);
		gLua->CheckType(2, GLua::TYPE_NUMBER);

		IITTrackCollection *iTrackCollection = (IITTrackCollection*)gLua->GetUserData(1);

		IITTrack *iTrack = NULL;

		if (iTrackCollection->get_Item(gLua->GetInteger(2), &iTrack) == S_OK)
			gLua->PushUserData(gLua->GetMetaTable(IITTRACK_NAME, IITTRACK_ID), iTrack);
		else
			gLua->PushNil();

		return 1;
	}

	LUA_FUNCTION(GetItemByName)
	{
		gLua->CheckType(1, IITTRACKCOLLECTION_ID);
		gLua->CheckType(2, GLua::TYPE_STRING);

		IITTrackCollection *iTrackCollection = (IITTrackCollection*)gLua->GetUserData(1);

		CComBSTR name(gLua->GetString(2));

		IITTrack *iTrack = NULL;

		if (iTrackCollection->get_ItemByName(name, &iTrack) == S_OK)
			gLua->PushUserData(gLua->GetMetaTable(IITTRACK_NAME, IITTRACK_ID), iTrack);
		else
			gLua->PushNil();

		return 1;
	}
}

namespace IITPlaylistMethods
{
	LUA_FUNCTION(Delete)
	{
		gLua->CheckType(1, IITPLAYLIST_ID);

		IITPlaylist *iPlaylist = (IITPlaylist*)gLua->GetUserData(1);

		if (iPlaylist->Delete() == S_OK)
			gLua->Push(true);
		else
			gLua->Push(false);

		return 1;
	}
	
	LUA_FUNCTION(GetDuration)
	{
		gLua->CheckType(1, IITPLAYLIST_ID);

		IITPlaylist *iPlaylist = (IITPlaylist*)gLua->GetUserData(1);

		long duration;
		
		if (iPlaylist->get_Duration(&duration) == S_OK)
			gLua->PushLong(duration);
		else
			gLua->PushNil();

		return 1;
	}

	LUA_FUNCTION(GetKind)
	{
		gLua->CheckType(1, IITPLAYLIST_ID);

		IITPlaylist *iPlaylist = (IITPlaylist*)gLua->GetUserData(1);

		ITPlaylistKind kind;

		if (iPlaylist->get_Kind(&kind) == S_OK)
			gLua->PushDouble(kind);
		else
			gLua->PushNil();

		return 1;
	}

	LUA_FUNCTION(GetShuffle)
	{
		gLua->CheckType(1, IITPLAYLIST_ID);

		IITPlaylist *iPlaylist = (IITPlaylist*)gLua->GetUserData(1);

		short isShuffle;

		if (iPlaylist->get_Shuffle(&isShuffle) == S_OK)
			gLua->Push((isShuffle == VARIANT_TRUE));
		else
			gLua->PushNil();

		return 1;
	}

	LUA_FUNCTION(GetSize)
	{
		gLua->CheckType(1, IITPLAYLIST_ID);

		IITPlaylist *iPlaylist = (IITPlaylist*)gLua->GetUserData(1);

		double size;

		if (iPlaylist->get_Size(&size) == S_OK)
			gLua->PushDouble(size);
		else
			gLua->PushNil();

		return 1;
	}

	LUA_FUNCTION(GetSongRepeat)
	{
		gLua->CheckType(1, IITPLAYLIST_ID);

		IITPlaylist *iPlaylist = (IITPlaylist*)gLua->GetUserData(1);

		ITPlaylistRepeatMode repeatMode;

		if (iPlaylist->get_SongRepeat(&repeatMode) == S_OK)
			gLua->PushDouble(repeatMode);
		else
			gLua->PushNil();

		return 1;
	}

	LUA_FUNCTION(GetSource)
	{
		gLua->CheckType(1, IITPLAYLIST_ID);

		IITPlaylist *iPlaylist = (IITPlaylist*)gLua->GetUserData(1);

		IITSource *iSource = NULL;

		if (iPlaylist->get_Source(&iSource) == S_OK)
			gLua->PushUserData(gLua->GetMetaTable(IITSOURCE_NAME, IITSOURCE_ID), iSource);
		else
			gLua->PushNil();

		return 1;
	}

	LUA_FUNCTION(GetTime)
	{
		gLua->CheckType(1, IITPLAYLIST_ID);

		IITPlaylist *iPlaylist = (IITPlaylist*)gLua->GetUserData(1);
		
		CComBSTR time;

		if (iPlaylist->get_Time(&time) == S_OK)
			gLua_PushBSTR(time);
		else
			gLua->PushNil();

		return 1;
	}

	LUA_FUNCTION(GetTracks)
	{
		gLua->CheckType(1, IITPLAYLIST_ID);

		IITPlaylist *iPlaylist = (IITPlaylist*)gLua->GetUserData(1);

		IITTrackCollection *iTrackCollection = NULL;

		if (iPlaylist->get_Tracks(&iTrackCollection) == S_OK)
			gLua->PushUserData(gLua->GetMetaTable(IITTRACKCOLLECTION_NAME, IITTRACKCOLLECTION_ID), iTrackCollection);
		else
			gLua->PushNil();

		return 1;
	}

	LUA_FUNCTION(GetVisible)
	{
		gLua->CheckType(1, IITPLAYLIST_ID);

		IITPlaylist *iPlaylist = (IITPlaylist*)gLua->GetUserData(1);

		short isVisible;

		if (iPlaylist->get_Visible(&isVisible) == S_OK)
			gLua->Push((bool)isVisible);
		else
			gLua->PushNil();

		return 1;
	}

	LUA_FUNCTION(PlayFirstTrack)
	{
		gLua->CheckType(1, IITPLAYLIST_ID);

		IITPlaylist *iPlaylist = (IITPlaylist*)gLua->GetUserData(1);

		if (iPlaylist->PlayFirstTrack() == S_OK)
			gLua->Push(true);
		else
			gLua->Push(false);

		return 1;
	}

	LUA_FUNCTION(Search)
	{
		gLua->CheckType(1, IITPLAYLIST_ID);
		gLua->CheckType(2, GLua::TYPE_STRING);
		gLua->CheckType(3, GLua::TYPE_NUMBER);

		IITPlaylist *iPlaylist = (IITPlaylist*)gLua->GetUserData(1);

		CComBSTR searchText(gLua->GetString(2));
		
		ITPlaylistSearchField searchFields = static_cast<ITPlaylistSearchField>(gLua->GetInteger(3));

		IITTrackCollection *iTrackCollection = NULL;

		if (iPlaylist->Search(searchText, searchFields, &iTrackCollection) == S_OK)
			gLua->PushUserData(gLua->GetMetaTable(IITTRACKCOLLECTION_NAME, IITTRACKCOLLECTION_ID), iTrackCollection);
		else
			gLua->PushNil();

		return 1;
	}

	LUA_FUNCTION(SetShuffle)
	{
		gLua->CheckType(1, IITPLAYLIST_ID);
		gLua->CheckType(2, GLua::TYPE_BOOL);

		IITPlaylist *iPlaylist = (IITPlaylist*)gLua->GetUserData(1);

		short shouldShuffle = VARIANT_FALSE;

		if (gLua->GetBool(2))
			shouldShuffle = VARIANT_TRUE;

		if (iPlaylist->put_Shuffle(shouldShuffle) == S_OK)
			gLua->Push(true);
		else
			gLua->Push(false);
		
		return 1;
	}

	LUA_FUNCTION(SetSongRepeat)
	{
		gLua->CheckType(1, IITPLAYLIST_ID);
		gLua->CheckType(2, GLua::TYPE_NUMBER);

		IITPlaylist *iPlaylist = (IITPlaylist*)gLua->GetUserData(1);

		if (iPlaylist->put_SongRepeat(static_cast<ITPlaylistRepeatMode>(gLua->GetInteger(2))) == S_OK)
			gLua->Push(true);
		else
			gLua->Push(false);

		return 1;
	}
}

namespace IITPlaylistCollectionMethods
{
	LUA_FUNCTION(GetCount)
	{
		gLua->CheckType(1, IITPLAYLISTCOLLECTION_ID);

		IITPlaylistCollection *iPlaylistCollection = (IITPlaylistCollection*)gLua->GetUserData(1);
	
		long count;

		if (iPlaylistCollection->get_Count(&count) == S_OK)
			gLua->PushLong(count);
		else
			gLua->PushNil();

		return 1;
	}

	LUA_FUNCTION(GetItem)
	{
		gLua->CheckType(1, IITPLAYLISTCOLLECTION_ID);
		gLua->CheckType(2, GLua::TYPE_NUMBER);

		IITPlaylistCollection *iPlaylistCollection = (IITPlaylistCollection*)gLua->GetUserData(1);
	
		IITPlaylist *iPlaylist = NULL;

		if (iPlaylistCollection->get_Item(gLua->GetInteger(2), &iPlaylist) == S_OK)
			gLua->PushUserData(gLua->GetMetaTable(IITPLAYLIST_NAME, IITPLAYLIST_ID), iPlaylist);
		else
			gLua->PushNil();

		return 1;
	}

	LUA_FUNCTION(GetItemByName)
	{
		gLua->CheckType(1, IITPLAYLISTCOLLECTION_ID);
		gLua->CheckType(2, GLua::TYPE_STRING);

		IITPlaylistCollection *iPlaylistCollection = (IITPlaylistCollection*)gLua->GetUserData(1);
	
		CComBSTR name(gLua->GetString(2));

		IITPlaylist *iPlaylist = NULL;

		if (iPlaylistCollection->get_ItemByName(name, &iPlaylist) == S_OK)
			gLua->PushUserData(gLua->GetMetaTable(IITPLAYLIST_NAME, IITPLAYLIST_ID), iPlaylist);
		else
			gLua->PushNil();

		return 1;
	}
}

namespace IITSourceMethods
{
	LUA_FUNCTION(GetCapacity)
	{
		gLua->CheckType(1, IITSOURCE_ID);

		IITSource *iSource = (IITSource*)gLua->GetUserData(1);
		
		double capacity;

		if (iSource->get_Capacity(&capacity) == S_OK)
			gLua->PushDouble(capacity);
		else
			gLua->PushNil();

		return 1;
	}

	LUA_FUNCTION(GetFreeSpaceI)
	{
		gLua->CheckType(1, IITSOURCE_ID);

		IITSource *iSource = (IITSource*)gLua->GetUserData(1);
	
		double freespace;

		if (iSource->get_FreeSpace(&freespace) == S_OK)
			gLua->PushDouble(freespace);
		else
			gLua->PushNil();

		return 1;
	}

	LUA_FUNCTION(GetKind)
	{
		gLua->CheckType(1, IITSOURCE_ID);

		IITSource *iSource = (IITSource*)gLua->GetUserData(1);

		ITSourceKind kind;

		if (iSource->get_Kind(&kind) == S_OK)
			gLua->PushDouble(kind);
		else
			gLua->PushNil();

		return 1;
	}

	LUA_FUNCTION(GetPlaylists)
	{
		gLua->CheckType(1, IITSOURCE_ID);

		IITSource *iSource = (IITSource*)gLua->GetUserData(1);

		IITPlaylistCollection *iPlaylistCollection = NULL;

		if (iSource->get_Playlists(&iPlaylistCollection) == S_OK)
			gLua->PushUserData(gLua->GetMetaTable(IITPLAYLISTCOLLECTION_NAME, IITPLAYLISTCOLLECTION_ID), iPlaylistCollection);
		else
			gLua->PushNil();

		return 1;
	}
}

namespace IITSourceCollectionMethods
{
	LUA_FUNCTION(GetCount)
	{
		gLua->CheckType(1, IITSOURCECOLLECTION_ID);

		IITSourceCollection *iSourceCollection = (IITSourceCollection*)gLua->GetUserData(1);

		long count;

		if (iSourceCollection->get_Count(&count) == S_OK)
			gLua->PushLong(count);
		else
			gLua->PushNil();

		return 1;
	}

	LUA_FUNCTION(GetItem)
	{
		gLua->CheckType(1, IITSOURCECOLLECTION_ID);
		gLua->CheckType(2, GLua::TYPE_NUMBER);

		IITSourceCollection *iSourceCollection = (IITSourceCollection*)gLua->GetUserData(1);

		IITSource *iSource = NULL;

		if (iSourceCollection->get_Item(gLua->GetInteger(2), &iSource) == S_OK)
			gLua->PushUserData(gLua->GetMetaTable(IITSOURCE_NAME, IITSOURCE_ID), iSource);
		else
			gLua->PushNil();

		return 1;
	}

	LUA_FUNCTION(GetItemByName)
	{
		gLua->CheckType(1, IITSOURCECOLLECTION_ID);
		gLua->CheckType(2, GLua::TYPE_STRING);

		IITSourceCollection *iSourceCollection = (IITSourceCollection*)gLua->GetUserData(1);

		CComBSTR name(gLua->GetString(2));

		IITSource *iSource = NULL;
		
		if (iSourceCollection->get_ItemByName(name, &iSource) == S_OK)
			gLua->PushUserData(gLua->GetMetaTable(IITSOURCE_NAME, IITSOURCE_ID), iSource);
		else
			gLua->PushNil();

		return 1;
	}
}

LUA_FUNCTION(CreateInterface)
{
	IiTunes *iITunes = NULL;

	if (CoCreateInstance(CLSID_iTunesApp, NULL, CLSCTX_LOCAL_SERVER, IID_IiTunes, (void**)&iITunes) == S_OK)
		gLua->PushUserData(gLua->GetMetaTable(IITUNES_NAME, IITUNES_ID), iITunes);
	else
		gLua->PushNil();

	return 1;
}

int Load(lua_State *L)
{
	gLua = Lua();
	
	ILuaObject *__index = NULL;

	ILuaObject *iTunesMT = gLua->GetMetaTable(IITUNES_NAME, IITUNES_ID);
		__index = gLua->GetNewTable();
			__index->SetMember("Release", IDispatchMethods::Release);
	
			__index->SetMember("BackTrack", IiTunesMethods::BackTrack);
			__index->SetMember("FastForward", IiTunesMethods::FastForward);
			__index->SetMember("GetCurrentStreamTitle", IiTunesMethods::GetCurrentStreamTitle);
			__index->SetMember("GetCurrentStreamURL", IiTunesMethods::GetCurrentStreamURL);
			__index->SetMember("GetCurrentPlaylist", IiTunesMethods::GetCurrentPlaylist);
			__index->SetMember("GetCurrentTrack", IiTunesMethods::GetCurrentTrack);
			__index->SetMember("GetLibraryPlaylist", IiTunesMethods::GetLibraryPlaylist);
			__index->SetMember("GetLibrarySource", IiTunesMethods::GetLibrarySource);
			__index->SetMember("GetMute", IiTunesMethods::GetMute);
			__index->SetMember("GetPlayerPosition", IiTunesMethods::GetPlayerPosition);
			__index->SetMember("GetPlayerState", IiTunesMethods::GetPlayerState);
			__index->SetMember("GetSoundVolume", IiTunesMethods::GetSoundVolume);
			__index->SetMember("GetSources", IiTunesMethods::GetSources);
			__index->SetMember("GetVersion", IiTunesMethods::GetVersion);
			__index->SetMember("NextTrack", IiTunesMethods::NextTrack);
			__index->SetMember("Pause", IiTunesMethods::Pause);
			__index->SetMember("Play", IiTunesMethods::Play);
			__index->SetMember("PlayFile", IiTunesMethods::PlayFile);
			__index->SetMember("PlayPause", IiTunesMethods::PlayPause);
			__index->SetMember("PreviousTrack", IiTunesMethods::PreviousTrack);
			__index->SetMember("Quit", IiTunesMethods::Quit);
			__index->SetMember("Resume", IiTunesMethods::Resume);
			__index->SetMember("Rewind", IiTunesMethods::Rewind);
			__index->SetMember("SetMute", IiTunesMethods::SetMute);
			__index->SetMember("SetPlayerPosition", IiTunesMethods::SetPlayerPosition);
			__index->SetMember("SetSoundVolume", IiTunesMethods::SetSoundVolume);
			__index->SetMember("Stop", IiTunesMethods::Stop);
			__index->SetMember("SubscribeToPodcast", IiTunesMethods::SubscribeToPodcast);
			__index->SetMember("UpdateIPod", IiTunesMethods::UpdateIPod);
			__index->SetMember("UpdatePodcastFeeds", IiTunesMethods::UpdatePodcastFeeds);

			iTunesMT->SetMember("__index", __index);

			iTunesMT->SetMember("__gc", IDispatchMethods::Release);
		__index->UnReference();
	iTunesMT->UnReference();

	ILuaObject *IITTrackMT = gLua->GetMetaTable(IITTRACK_NAME, IITTRACK_ID);
		__index = gLua->GetNewTable();
			__index->SetMember("Release", IDispatchMethods::Release);

			__index->SetMember("GetIndex", IITObjectMethods::GetIndex);
			__index->SetMember("GetName", IITObjectMethods::GetName);
			__index->SetMember("GetPlaylistID", IITObjectMethods::GetPlaylistID);
			__index->SetMember("GetSourceID", IITObjectMethods::GetSourceID);
			__index->SetMember("GetTrackDatabaseID", IITObjectMethods::GetTrackDatabaseID);
			__index->SetMember("GetTrackID", IITObjectMethods::GetTrackID);
			__index->SetMember("SetName", IITObjectMethods::SetName);

			__index->SetMember("Delete", IITTrackMethods::Delete);
			__index->SetMember("GetAlbum", IITTrackMethods::GetAlbum);
			__index->SetMember("GetArtist", IITTrackMethods::GetArtist);
			__index->SetMember("GetBitRate", IITTrackMethods::GetBitRate);
			__index->SetMember("GetBPM", IITTrackMethods::GetBPM);
			__index->SetMember("GetComment", IITTrackMethods::GetComment);
			__index->SetMember("GetCompilation", IITTrackMethods::GetCompilation);
			__index->SetMember("GetComposer", IITTrackMethods::GetComposer);
			__index->SetMember("GetDateAdded", IITTrackMethods::GetDateAdded);
			__index->SetMember("GetDiscCount", IITTrackMethods::GetDiscCount);
			__index->SetMember("GetDiscNumber", IITTrackMethods::GetDiscNumber);
			__index->SetMember("GetDuration", IITTrackMethods::GetDuration);
			__index->SetMember("GetEnabled", IITTrackMethods::GetEnabled);
			__index->SetMember("GetEQ", IITTrackMethods::GetEQ);
			__index->SetMember("GetFinish", IITTrackMethods::GetFinish);
			__index->SetMember("GetGenre", IITTrackMethods::GetGenre);
			__index->SetMember("GetGrouping", IITTrackMethods::GetGrouping);
			__index->SetMember("GetKind", IITTrackMethods::GetKind);
			__index->SetMember("GetModificationDate", IITTrackMethods::GetModificationDate);
			__index->SetMember("GetPlayedCount", IITTrackMethods::GetPlayedCount);
			__index->SetMember("GetPlayedDate", IITTrackMethods::GetPlayedDate);
			__index->SetMember("GetPlaylist", IITTrackMethods::GetPlaylist);
			__index->SetMember("GetPlayOrderIndex", IITTrackMethods::GetPlayOrderIndex);
			__index->SetMember("GetRating", IITTrackMethods::GetRating);
			__index->SetMember("GetSampleRate", IITTrackMethods::GetSampleRate);
			__index->SetMember("GetSize", IITTrackMethods::GetSize);
			__index->SetMember("GetStart", IITTrackMethods::GetStart);
			__index->SetMember("GetTime", IITTrackMethods::GetTime);
			__index->SetMember("GetTrackCount", IITTrackMethods::GetTrackCount);
			__index->SetMember("GetTrackNumber", IITTrackMethods::GetTrackNumber);
			__index->SetMember("GetVolumeAdjustment", IITTrackMethods::GetVolumeAdjustment);
			__index->SetMember("GetYear", IITTrackMethods::GetYear);
			__index->SetMember("Play", IITTrackMethods::Play);
			__index->SetMember("SetAlbum", IITTrackMethods::SetAlbum);
			__index->SetMember("SetArtist", IITTrackMethods::SetArtist);
			__index->SetMember("SetBPM", IITTrackMethods::SetBPM);
			__index->SetMember("SetComment", IITTrackMethods::SetComment);
			__index->SetMember("SetCompilation", IITTrackMethods::SetCompilation);
			__index->SetMember("SetComposer", IITTrackMethods::SetComposer);
			__index->SetMember("SetDiscCount", IITTrackMethods::SetDiscCount);
			__index->SetMember("SetDiscNumber", IITTrackMethods::SetDiscNumber);
			__index->SetMember("SetEnabled", IITTrackMethods::SetEnabled);
			__index->SetMember("SetEQ", IITTrackMethods::SetEQ);
			__index->SetMember("SetFinish", IITTrackMethods::SetFinish);
			__index->SetMember("SetGenre", IITTrackMethods::SetGenre);
			__index->SetMember("SetGrouping", IITTrackMethods::SetGrouping);
			__index->SetMember("SetPlayedCount", IITTrackMethods::SetPlayedCount);
			__index->SetMember("SetPlayedDate", IITTrackMethods::SetPlayedDate);
			__index->SetMember("SetRating", IITTrackMethods::SetRating);
			__index->SetMember("SetStart", IITTrackMethods::SetStart);
			__index->SetMember("SetTrackCount", IITTrackMethods::SetTrackCount);
			__index->SetMember("SetTrackNumber", IITTrackMethods::SetTrackNumber);
			__index->SetMember("SetVolumeAdjustment", IITTrackMethods::SetVolumeAdjustment);
			__index->SetMember("SetYear", IITTrackMethods::SetYear);

			IITTrackMT->SetMember("__index", __index);

			IITTrackMT->SetMember("__gc", IDispatchMethods::Release);
		__index->UnReference();
	IITTrackMT->UnReference();

	ILuaObject *IITTrackCollectionMT = gLua->GetMetaTable(IITTRACKCOLLECTION_NAME, IITTRACKCOLLECTION_ID);
		__index = gLua->GetNewTable();
			__index->SetMember("Release", IDispatchMethods::Release);

			__index->SetMember("GetCount", IITTrackCollectionMethods::GetCount);
			__index->SetMember("GetItem", IITTrackCollectionMethods::GetItem);
			__index->SetMember("GetItemByName", IITTrackCollectionMethods::GetItemByName);

			IITTrackCollectionMT->SetMember("__index", __index);

			IITTrackCollectionMT->SetMember("__gc", IDispatchMethods::Release);
		__index->UnReference();
	IITTrackCollectionMT->UnReference();

	ILuaObject *IITPlaylistMT = gLua->GetMetaTable(IITPLAYLIST_NAME, IITPLAYLIST_ID);
		__index = gLua->GetNewTable();
			__index->SetMember("Release", IDispatchMethods::Release);

			__index->SetMember("GetIndex", IITObjectMethods::GetIndex);
			__index->SetMember("GetName", IITObjectMethods::GetName);
			__index->SetMember("GetPlaylistID", IITObjectMethods::GetPlaylistID);
			__index->SetMember("GetSourceID", IITObjectMethods::GetSourceID);
			__index->SetMember("SetName", IITObjectMethods::SetName);

			__index->SetMember("Delete", IITPlaylistMethods::Delete);
			__index->SetMember("GetDuration", IITPlaylistMethods::GetDuration);
			__index->SetMember("GetKind", IITPlaylistMethods::GetKind);
			__index->SetMember("GetShuffle", IITPlaylistMethods::GetShuffle);
			__index->SetMember("GetSize", IITPlaylistMethods::GetSize);
			__index->SetMember("GetSongRepeat", IITPlaylistMethods::GetSongRepeat);
			__index->SetMember("GetSource", IITPlaylistMethods::GetSource);
			__index->SetMember("GetTime", IITPlaylistMethods::GetTime);
			__index->SetMember("GetTracks", IITPlaylistMethods::GetTracks);
			__index->SetMember("GetVisible", IITPlaylistMethods::GetVisible);
			__index->SetMember("PlayFirstTrack", IITPlaylistMethods::PlayFirstTrack);
			__index->SetMember("Search", IITPlaylistMethods::Search);
			__index->SetMember("SetShuffle", IITPlaylistMethods::SetShuffle);
			__index->SetMember("SetSongRepeat", IITPlaylistMethods::SetSongRepeat);

			IITPlaylistMT->SetMember("__index", __index);

			IITPlaylistMT->SetMember("__gc", IDispatchMethods::Release);
		__index->UnReference();
	IITPlaylistMT->UnReference();
	
	ILuaObject *IITPlaylistCollectionMT = gLua->GetMetaTable(IITPLAYLISTCOLLECTION_NAME, IITPLAYLISTCOLLECTION_ID);
		__index = gLua->GetNewTable();
			__index->SetMember("Release", IDispatchMethods::Release);

			__index->SetMember("GetCount", IITPlaylistCollectionMethods::GetCount);
			__index->SetMember("GetItem", IITPlaylistCollectionMethods::GetItem);
			__index->SetMember("GetItemByName", IITPlaylistCollectionMethods::GetItemByName);

			IITPlaylistCollectionMT->SetMember("__index", __index);

			IITPlaylistCollectionMT->SetMember("__gc", IDispatchMethods::Release);
		__index->UnReference();
	IITPlaylistCollectionMT->UnReference();

	ILuaObject *IITSourceMT = gLua->GetMetaTable(IITSOURCE_NAME, IITSOURCE_ID);
		__index = gLua->GetNewTable();
			__index->SetMember("Release", IDispatchMethods::Release);

			__index->SetMember("GetIndex", IITObjectMethods::GetIndex);
			__index->SetMember("GetName", IITObjectMethods::GetName);
			__index->SetMember("GetSourceID", IITObjectMethods::GetSourceID);
			__index->SetMember("SetName", IITObjectMethods::SetName);

			__index->SetMember("GetCapacity", IITSourceMethods::GetCapacity);
			__index->SetMember("GetFreeSpace", IITSourceMethods::GetFreeSpaceI);
			__index->SetMember("GetKind", IITSourceMethods::GetKind);	
			__index->SetMember("GetPlaylists", IITSourceMethods::GetPlaylists);
			
			IITSourceMT->SetMember("__index", __index);

			IITSourceMT->SetMember("__gc", IDispatchMethods::Release);
		__index->UnReference();
	IITSourceMT->UnReference();

	ILuaObject *IITSourceCollectionMT = gLua->GetMetaTable(IITSOURCECOLLECTION_NAME, IITSOURCECOLLECTION_ID);
		__index = gLua->GetNewTable();
			__index->SetMember("Release", IDispatchMethods::Release);

			__index->SetMember("GetCount", IITSourceCollectionMethods::GetCount);
			__index->SetMember("GetItem", IITSourceCollectionMethods::GetItem);
			__index->SetMember("GetItemByName", IITSourceCollectionMethods::GetItemByName);

			IITSourceCollectionMT->SetMember("__index", __index);

			IITSourceCollectionMT->SetMember("__gc", IDispatchMethods::Release);
		__index->UnReference();
	IITSourceCollectionMT->UnReference();

	gLua->NewGlobalTable("itunes");

	ILuaObject *iTunesT = gLua->GetGlobal("itunes");
		iTunesT->SetMember("CreateInterface", CreateInterface);
	iTunesT->UnReference();

	return 0;
}

int Unload(lua_State *L)
{
	return 0;
}